﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using HeuristicLab.Persistence.Default.CompositeSerializers.Storable;
using HeuristicLab.Core;
using HeuristicLab.Common;

namespace HeuristicLab.HiveParallelEngine {
  /// <summary>
  /// Represents an engine that executes operations which can be executed in parallel on the hive
  /// </summary>
  [StorableClass]
  [Item("Hive Parallel Engine", "Engine for parallel execution on the hive.")]
  public class HiveEngine : Engine {
    private IOperator currentOperator;

    #region constructors and cloning
    public HiveEngine() { }
    [StorableConstructor]
    protected HiveEngine(bool deserializing) : base(deserializing) { }
    protected HiveEngine(HiveEngine original, Cloner cloner) : base(original, cloner) {
    }
    public override IDeepCloneable Clone(Cloner cloner) {
      return new HiveEngine(this, cloner);
    }
    #endregion

    protected override void ProcessNextOperation() {
      currentOperator = null;
      IOperation next = ExecutionStack.Pop();
      OperationCollection coll = next as OperationCollection;

      if (coll.Parallel) {
                
      } else {
        while (coll != null) {
          for (int i = coll.Count - 1; i >= 0; i--)
            ExecutionStack.Push(coll[i]);
          next = ExecutionStack.Count > 0 ? ExecutionStack.Pop() : null;
          coll = next as OperationCollection;
        }
      }

      IAtomicOperation operation = next as IAtomicOperation;
      if (operation != null) {
        try {
          currentOperator = operation.Operator;
          ExecutionStack.Push(operation.Operator.Execute((IExecutionContext)operation));
        }
        catch (Exception ex) {
          ExecutionStack.Push(operation);
          OnExceptionOccurred(new OperatorExecutionException(operation.Operator, ex));
          Pause();
        }
        if (operation.Operator.Breakpoint) {
          Log.LogMessage(string.Format("Breakpoint: {0}", operation.Operator.Name != string.Empty ? operation.Operator.Name : operation.Operator.ItemName));
          Pause();
        }
      }
    }

    public override void Pause() {
      base.Pause();
      if (currentOperator != null) currentOperator.Abort();
    }
    public override void Stop() {
      base.Stop();
      if (currentOperator != null) currentOperator.Abort();
    }
  }
}
